unit ClockClient2Impl;

interface

uses
  ComObj;

type
  TClockSink = class(TInterfacedObject, IUnknown, IDispatch)
  protected
    //IUnknown
    //Method resolution clause to allow QueryInterface to be redefined
    function IUnknown.QueryInterface = QueryInterface;
    function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
    //IDispatch
    function GetTypeInfoCount(out Count: Integer): HResult; stdcall;
    function GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult; stdcall;
    function GetIDsOfNames(const IID: TGUID; Names: Pointer;
      NameCount, LocaleID: Integer; DispIDs: Pointer): HResult; stdcall;
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
      Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; stdcall;
  end;

implementation

uses
  Windows, ActiveX, Clock_TLB, Dialogs, ClockClient2MainForm, SysUtils;

{ TClockSink }

function TClockSink.GetIDsOfNames(const IID: TGUID; Names: Pointer;
  NameCount, LocaleID: Integer; DispIDs: Pointer): HResult;
begin
  Result := E_NOTIMPL
end;

function TClockSink.GetTypeInfo(Index, LocaleID: Integer;
  out TypeInfo): HResult;
begin
  Result := E_NOTIMPL
end;

function TClockSink.GetTypeInfoCount(out Count: Integer): HResult;
begin
  Count := 0;
  Result := S_OK
end;

function TClockSink.Invoke(DispID: Integer; const IID: TGUID;
  LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo,
  ArgErr: Pointer): HResult;
var
  Args: PVariantArgList;
  EventSender: IApplication;
  AlarmDateTime: TDateTime;
begin
  Result := S_OK;
  //This is called to trigger an event interface method, if implemented
  //We need to check which one it is (by DispID) and do something sensible if we
  //support the triggered event

  //Both event methods happen to have the same parameters,
  //so we can extract them just once to save duplication
  Args := TDispParams(Params).rgvarg;
  //Params are in reverse order:
  //Last parameter is at pos. 0
  //First parameter is at pos. cArgs - 1
  EventSender := IUnknown(OleVariant(Args^[1])) as IApplication;
  AlarmDateTime := OleVariant(Args^[0]);
  case DispID of
    1: {AlarmRing}
      ClockClientForm.Bar.Panels[2].Text := Format(
        'Alarm rang at %s. Time is now %s',
        [DateTimeToStr(AlarmDateTime),
         TimeToStr(EventSender.CurrentDateTime)]);
    2: {AlarmSet}
      ClockClientForm.Bar.Panels[2].Text := Format(
        'Alarm set for %s. Time is now %s',
        [DateTimeToStr(AlarmDateTime),
         TimeToStr(EventSender.CurrentDateTime)])
  else
    //Ignore other events
  end
end;

function TClockSink.QueryInterface(const IID: TGUID; out Obj): HResult;
begin
  Result := E_NOINTERFACE;
  //If events interface requested, return IDispatch
  if IsEqualIID(IID, DIID_IApplicationEvents) then
  begin
    if GetInterface(IDispatch, Obj) then
      Result := S_OK
  end
  else
    //Handle other interface requests normally
    if GetInterface(IID, Obj) then
      Result := S_OK
end;

end.
