unit ClockClient3MainForm;

interface

uses
  Clock_TLB,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls, StdCtrls;

type
  TClockClientForm = class(TForm)
    btnSetAlarm: TButton;
    Timer1: TTimer;
    Bar: TStatusBar;
    lblMsg: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure btnSetAlarmClick(Sender: TObject);
  private
    FSinkCookie: Integer;
  public
    ClockServer: IApplication;
    procedure AlarmClockRing(Clock: IApplication; AlarmDateTime: TDateTime);
    procedure AlarmClockSet(Clock: IApplication; AlarmDateTime: TDateTime);
  end;

var
  ClockClientForm: TClockClientForm;

implementation

{$R *.DFM}

uses
  ComObj, ClockClient3Impl;

procedure TClockClientForm.FormCreate(Sender: TObject);
const
  ProgID = 'Clock.Application';
var
  Sink: TClockSink;
begin
  try
    ClockServer := GetActiveOleObject(ProgID) as IApplication;
  except
    ClockServer := CreateOleObject(ProgID) as IApplication;
  end;
  //Create event sink
  Sink := TClockSink.Create;
  //Set up event sink event handlers
  Sink.OnAlarmRing := AlarmClockRing;
  Sink.OnAlarmSet := AlarmClockSet;
  //Connect to server
  InterfaceConnect(ClockServer, IApplicationEvents, Sink, FSinkCookie)
end;

procedure TClockClientForm.FormDestroy(Sender: TObject);
begin
  InterfaceDisconnect(ClockServer, IApplicationEvents, FSinkCookie);
end;

procedure TClockClientForm.Timer1Timer(Sender: TObject);
begin
  try
    Bar.Panels[0].Text := DateTimeToStr(ClockServer.CurrentDateTime);
    if ClockServer.AlarmSet then
      Bar.Panels[1].Text := 'Alarm set'
    else
      Bar.Panels[1].Text := 'Alarm not set';
  except
    Bar.Panels[0].Text := 'N/A';
    Bar.Panels[1].Text := 'N/A';
  end
end;

procedure TClockClientForm.btnSetAlarmClick(Sender: TObject);
begin
  if not ClockServer.AlarmSet then
    ClockServer.Alarm := Now + (5 / SecsPerDay)
  else
    ShowMessage('Alarm already set!')
end;

procedure TClockClientForm.AlarmClockRing(Clock: IApplication;
  AlarmDateTime: TDateTime);
var
  I: Integer;
begin
  Bar.Panels[2].Text := Format(
    'Alarm rang at %s. Time is now %s',
    [DateTimeToStr(AlarmDateTime),
     TimeToStr(Clock.CurrentDateTime)]);
  lblMsg.Caption := 'Ring!!!';
  for I := 0 to 8 do
  begin
    lblMsg.Font.Size := Succ(I mod 3) * 8;
    lblMsg.Update;
    Sleep(100)
  end;
  lblMsg.Caption := '';
end;

procedure TClockClientForm.AlarmClockSet(Clock: IApplication;
  AlarmDateTime: TDateTime);
begin
  Bar.Panels[2].Text := Format(
    'Alarm set for %s. Time is now %s',
    [DateTimeToStr(AlarmDateTime),
     TimeToStr(Clock.CurrentDateTime)])
end;

end.
