unit ClockImpl;

interface

uses
  ComObj, ActiveX, AxCtrls, Classes, Clock_TLB, StdVcl,
  ExtCtrls, SysUtils, Windows;

type
  TApplication = class(TAutoObject, IConnectionPointContainer, IApplication)
  private
    { Private declarations }
    FConnectionPoints: TConnectionPoints;
    FConnectionPoint: TConnectionPoint;
    FSinkList: TList;
    FEvents: IApplicationEvents;
    //New stuff
    FAlarmTime: TDateTime;
    FTimer: TTimer;
    FROTCookie: Longint;
    procedure TimerTick(Sender: TObject);
  public
    procedure Initialize; override;
    destructor Destroy; override;
  protected
    { Protected declarations }
    property ConnectionPoints: TConnectionPoints read FConnectionPoints
      implements IConnectionPointContainer;
    procedure EventSinkChanged(const EventSink: IUnknown); override;
    function Get_Alarm: TDateTime; safecall;
    function Get_AlarmSet: WordBool; safecall;
    function Get_CurrentDateTime: TDateTime; safecall;
    procedure Set_Alarm(Value: TDateTime); safecall;
  end;

implementation

uses ComServ;

procedure TApplication.EventSinkChanged(const EventSink: IUnknown);
begin
  FEvents := EventSink as IApplicationEvents;
  if FConnectionPoint <> nil then
     FSinkList := FConnectionPoint.SinkList;
end;

procedure TApplication.Initialize;
begin
  inherited Initialize;
  //Register object in ROT
  OleCheck(RegisterActiveObject(Self, CLASS_Application, ActiveObject_Weak, FROTCookie));
  //Set up connection points manager and connection point
  FConnectionPoints := TConnectionPoints.Create(Self);
  if AutoFactory.EventTypeInfo <> nil then
    FConnectionPoint := FConnectionPoints.CreateConnectionPoint(
      AutoFactory.EventIID, ckMulti, EventConnect)
  else FConnectionPoint := nil;
end;

destructor TApplication.Destroy;
begin
  FTimer.Free;
  //Remove object from ROT
  OleCheck(RevokeActiveObject(FROTCookie, nil));
  inherited;
end;

function TApplication.Get_Alarm: TDateTime;
begin
  if Get_AlarmSet then
    Result := FAlarmTime
  else
    raise EOleSysError.Create(
      'Alarm is not set', //error message
      MakeResult(SEVERITY_ERROR, FACILITY_ITF, 1), //custom interface error
      0) //help context
end;

function TApplication.Get_AlarmSet: WordBool;
begin
  //Alarm is set if the timer has been created and is enabled
  Result := Assigned(FTimer) and FTimer.Enabled
end;

function TApplication.Get_CurrentDateTime: TDateTime;
begin
  Result := Now
end;

procedure TApplication.Set_Alarm(Value: TDateTime);
var
  I: Integer;
begin
  if not Assigned(FTimer) then
  begin
     FTimer := TTimer.Create(nil);
     FTimer.Interval := 100;
     FTimer.Enabled := False;
     FTimer.OnTimer := TimerTick
  end;
  FAlarmTime := Value;
  FTimer.Enabled := True;
  //Trigger alarm set event
  if Assigned(FSinkList) then
    for I := 0 to FSinkList.Count - 1 do
      if Assigned(FSinkList[I]) then
        (IUnknown(FSinkList[I]) as IApplicationEvents).AlarmSet(Self, FAlarmTime);
end;

procedure TApplication.TimerTick(Sender: TObject);
var
  I: Integer;
begin
  if Now >= FAlarmTime then
  begin
    FTimer.Enabled := False;
    //Trigger alarm ring event
    if Assigned(FSinkList) then
      for I := 0 to FSinkList.Count - 1 do
        if Assigned(FSinkList[I]) then
          (IUnknown(FSinkList[I]) as IApplicationEvents).AlarmRing(Self, FAlarmTime);
  end
end;

initialization
  TAutoObjectFactory.Create(ComServer, TApplication, Class_Application,
    ciMultiInstance, tmApartment);
end.
